#!/usr/bin/env bash

# exit when any command fails, echo commands
set -ex

# In docker container, run as follows:
# DISPLAY= database_engine=filebased LD_LIBRARY_PATH=/home/micado/.conda/envs/micadowise/lib bash build.sh --delete
# - Disable DISPLAY, since the figure creation tries to access X.
# - Turn off database, since there might be database evolution necessary.
# - Set LD_LIBRARY_PATH to prevent some linking problems.
# - --delete to delete auto-generated files.

# TODO: Convert into Makefile?

# Parse command line options, --delete will delete auto-generated files.
while [[ $# -gt 0 ]]; do
  case $1 in
    --delete)
      DELETE=YES
      shift # past argument
      ;;
    *)
      POSITIONAL_ARGS+=("$1") # save positional arg
      shift # past argument
      ;;
  esac
done

# TODO: Ensure conda environment is okay.
# conda install openjdk

#if ! command -v pajv; then
#    npm install -g pajv
#fi

# Ensure that the MicadoWISE code in the current directory is used.
export PYTHONPATH="${PYTHONPATH-$(pwd)/src}"

if [ ! -f ditaa-0.11.0-standalone.jar ]; then
    wget https://github.com/stathissideris/ditaa/releases/download/v0.11.0/ditaa-0.11.0-standalone.jar
fi

if [ ! -f vodslparser-0.4.0.jar ]; then
    wget https://repo1.maven.org/maven2/org/javastro/vodsl/vodslparser/0.4.0/vodslparser-0.4.0.jar
fi

pushd src/vodml || exit
    python finderrors.py
    java -jar ../../vodslparser-0.4.0.jar micado.vodsl
    sed "s|<lastModified>.*</lastModified>|<lastModified>1970-01-01T00:00:00Z</lastModified>|" -i micado.vo-dml.xml
popd || exit

pip install git+https://github.com/xyz2tex/svg2tikz.git@7a9959c295e1ed73e543474c6f3679d04cebc9e9

# Remove auto-generated files. Enable with --delete
if [ "${DELETE}" = "YES" ] ; then
    echo "Going to delete auto-generated files!"
    # Double quotes in center of string to prevent removal of this file itself.
    rm -f $(rg "This file is automatically generated."" Do not modify!" -l)
    rm -f docs/source/generated/*.tex docs/source/generated/*.svg docs/source/generated/*.ditaa docs/source/generated/*.rst docs/source/generated/*.png
    rm -f src/micado/templates/*.tsf
fi

python src/micado/toolbox/yaml_from_did.py
python src/micado/toolbox/python_from_vodml.py
black src/micado
python src/micado/toolbox/tsf_from_python.py
python src/micado/toolbox/obx_from_python.py
python src/micado/toolbox/rst_from_python.py
python src/micado/toolbox/tex_from_python.py
python src/micado/toolbox/ditaa_from_wise.py
python src/micado/toolbox/create_associationmatrix.py
python src/micado/toolbox/oca_from_python.py > reflex/micado.oca
python src/micado/toolbox/check_yaml_extra.py

# Convert Jupyter notebooks to regular Python files
for fn_nb in notebooks/*.ipynb; do
    jupytext "${fn_nb}" --to py
done

# Copy generated files that tests are written against. It is up to the developer
# to ensure that the changes are correct before committing.
cp docs/source/generated/MasterDark.rst src/micado/test/functional/MasterDark.rst
cp docs/source/generated/MasterDark.tex src/micado/test/functional/MasterDark.tex
cp docs/source/generated/Detrend.tex src/micado/test/functional/Detrend.tex
cp docs/source/generated/Detrend.ditaa src/micado/test/functional/
cp docs/source/generated/associationmatrix.ditaa src/micado/test/functional/associationmatrix.ditaa
cp docs/source/generated/associationmatrix_hci.ditaa src/micado/test/functional/associationmatrix_hci.ditaa
cp reflex/micado.oca src/micado/test/functional/

# Copy manual ditaa figures that do need to be converted to svg and tikz.
# Having the manual figures in another directory allows the directory with
# generated files to be deleted at will. Ditaa files must not have extra
# periods . in their name because the tikz conversion expects only one.
# associationmatrix_manual.ditaa has 3 changes w.r.t. associationmatrix.ditaa
# - The horizontal lines between output products to input products are
#   deduplicated. E.g. between the DEPERSISTED, DETREND and DETRENDFINAL.
cp docs/source/images/*.ditaa docs/source/generated/

for fn_ditaa in docs/source/generated/*.ditaa; do
    fn_base=$(basename "${fn_ditaa}" ".ditaa")
    fn_png="${fn_base}.png"
    fn_svg="${fn_base}.svg"
    #nice -n 15 java -jar ditaa-0.11.0-standalone.jar "${fn_ditaa}" -E --no-shadows "docs/source/generated/${fn_png}"
    nice -n 15 java -jar ditaa-0.11.0-standalone.jar "${fn_ditaa}" -E --no-shadows --svg "docs/source/generated/${fn_svg}"
done
wait
python src/micado/toolbox/tikz_from_svg.py

#pushd src/datainterface || exit
#pajv -s dicschema.hierarchical.json -d "yaml/*.hierarch.*" -m metaschema.json;
#pajv -s dicschema.flat.json -d "yaml/*.flat.*" -m metaschema.json;
#popd || exit

#pushd docs || exit
#make html
#popd || exit

