# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.13.1
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

# # <center>A guide through the</center>
# # <center>MICADO Standard Imaging Pipeline docs and prototype software</center>
# # <center>aka</center>
# # <center>*MICADO Standard Imaging Digital Design*</center>

# We present the ESO Final Design documentation and software prototype for the MICADO Standard Imaging data calibration plus data reduction pipeline. We use a single source of digital information from which we build the documentation and software prototype dynamically.

# **Technical preamble:**
#
# The easiest way to run this notebook is [through Docker](https://gitlab.astro-wise.org/micado/micadowise/-/blob/master/docker/README.md).
#
# The notebook can also be ran in a [standalone MicadoWISE installation](https://gitlab.astro-wise.org/micado/micadowise/-/blob/master/README.md). After installation run
# ```
# conda activate micadowise
# jupyter notebook
# ```
#
# The result should be that a Jupyter Python Notebook interactive user interface opens up in your browser. See this [Kapteyn howto](https://www.astro.rug.nl/KAPTEYN_DB/howto_view.php?stage=2&id=110&auth=0&order=&searchstr=jupyter&personselected=&group=comp) for details.
#
#
# Authors: Gijs A. Verdoes Kleijn (g.a.verdoes.kleijn@rug.nl) and Hugo Buddelmeijer (h.buddelmeijer@rug.nl)

# <a id = "Intended reader audience" IntendedAudience></a>
# ## Intended reader audience
#
# We intend that this Jupyter notebook is digestable for a person with the following qualifications:
# * interest in astronomical instrumentation, in particular the MICADO instrument
# * familiar with the Python programming language
# * has executed data calibration and data reduction software pipelines
#
# Please inform the authors if you experience the notebook demands that you possess additional qualifications.

# ## Introduction<a name="Introduction"></a>
#
#
# This notebook is a guide through the design of the standard imaging data calibration and reduction software for MICADO, the First Light near-IR instrument on the Extremely Large Telescope. It is a digital design in which the core is a machine readable version of the specification of the data items, instrument hardware, software recipes and pipeline. From this we derive a detailed design that is readable for humans. This design documentation, the prototype pipelines and recipes inside it, and data archives are generated in an au-
# tomated fashion. This approach keeps design and prototype implementation consistent.
#
# Credits: our approach is based on expertise and lessons learned in our team in earlier projects (e.g. OmegaCAM, MUSE, Euclid).

# <a id = "ESO Documentation" ESODocumentation></a>
# ## ESO Documentation
#
# If you are familiar with [ESO's data flow system](https://drive.google.com/file/d/1ZoDmp40CkGU3nZIqu7bqpCFdXCLaFhSX/view) for data pipelines for their instruments you might want to read the usual ESO documentation for the MICADO Standard Imaging pipeline:
#
# [Data Reduction Library Design Document](https://gitlab.astro-wise.org/api/v4/projects/micado%2Fdatareductionlibraryspecification/jobs/artifacts/master/raw/ELT-SPE-MCD-56305-0008_Data_Reduction_Library_Specification.pdf?job=latex_pdf)
#
# [Calibration Plan](https://gitlab.astro-wise.org/api/v4/projects/micado%2Fcalibrationplan/jobs/artifacts/master/raw/ELT-PLA-MCD-56305-0007_Calibration_Plan.pdf?job=latex_pdf)

# ## Quick-start user guide: a recipe execution example<a name="QuickStartUserGuide"></a>

# +
import matplotlib.pyplot as plt
# %matplotlib inline

from micado.hardware.ShotNoise import ShotNoise
from micado.main.ObservingBlock import ObservingBlock
from micado.hardware.Detector import Detector
from micado.hardware.DetectorArray import DetectorArray
from micado.hardware.DarkCurrent import DarkCurrent
from micado.hardware.SummedExposure import SummedExposure
from micado.dataitems.Dark import Dark
from micado.dataitems.Dark import DarkExtension
from micado.dataitems.DepersistedDark import DepersistedDark
from micado.dataitems.MasterDark import MasterDark
from micado.dataitems.MasterDark import MasterDarkExtension
from micado.main.Imstat import Imstat
from micado.requirements.DPSmasterdark import DPSmasterdark
from micado.templates.MicadoImgCalDark import MicadoImgCalDark
from micado.recipes.McdDetDark import McdDetDark
from micado.toolbox.ditaa_from_wise import ditaa_from_wise

from micado.test.functional.masterdark_test import mock_depersist
# -

# ### Setup the instrument

# +
# The size of the detector in pixel.
height_width = 32

# The value of the Dark current.
dark_real = 100.

# Model a single Detector.
mydetector = Detector(
    x_size=height_width,
    y_size=height_width,
)
mydetectorlist = [mydetector]

# Model the dark current.
mydarkcurrent = DarkCurrent(value=dark_real)

# Model the shotnoise.
myshotnoise = ShotNoise(state=1)

# Model the detector array.
mydetectorarray = DetectorArray(
    name='testdetector',
    detector_array_list=mydetectorlist,
    dark_current=mydarkcurrent,
    shot_noise=myshotnoise,
    qe_curve=None,
    linearity=None,
    readout_noise=None,
    exposure_action=SummedExposure(),
)
# -

# ### Simulate a raw Dark

# +
# Create a template to simulate.
dark_template = MicadoImgCalDark(det__dit=1, det__ndit=1)

# Create an observingblock using this template.
dark_ob = ObservingBlock(
    name="MyDarkTest",
    instrument="MICADO",
    target="empty_sky",
    templates=[dark_template],
)

# Initialize DarkExtensions
myextensions = [
    DarkExtension(detector=det, BITPIX=-64, NAXIS=2)
    for det in mydetectorlist
]

# Create raw Darks.
mydark = Dark(
    detector=mydetectorarray,
    template=dark_template,
    observing_block=dark_ob,
    extensions=myextensions,
)
mydark.make_simulation()

# -

print(mydark.extensions[0].image)
print(mydark.extensions[0].image.mean(), mydark.extensions[0].image.std())

plt.imshow(mydark.extensions[0].image, cmap='gray')
plt.colorbar()

# ### Simulate Several Raw Darks

mydarks = []
mydepersisteddarks = []
for _ in range(10):
    myextensions = [
        DarkExtension(detector=det, BITPIX=-64, NAXIS=2)
        for det in mydetectorlist
    ]
    mydark = Dark(
        detector=mydetectorarray,
        template=dark_template,
        observing_block=dark_ob,
        extensions=myextensions,
    )
    mydark.make_simulation()
    mydepersisteddark = mock_depersist(DepersistedDark, mydark)

    mydarks.append(mydark)
    mydepersisteddarks.append(mydepersisteddark)

# ### Creating a MasterDark

# +
# A recipe. The dark recipe has no parameters.
myrecipe = McdDetDark()

# Initialize FITS extensions.
mymasterextensions = [
    MasterDarkExtension(BITPIX=-64, NAXIS=2)
    for _ in myextensions
]

# Create MasterDark.
mymasterdark = MasterDark(
    recipe=myrecipe,
    darks=mydepersisteddarks,
    extensions=mymasterextensions,
)
mymasterdark.make_proto()

# -

print(mymasterdark.extensions[0].image)
print(mymasterdark.extensions[0].image.mean(), mymasterdark.extensions[0].image.std())

plt.imshow(mymasterdark.extensions[0].image, cmap='gray')
plt.colorbar()

# ### Verify MasterDark Requirements

# Verify that the measured dark current is the same as the real current.
myreq = DPSmasterdark()
myreq.precision = 0.05
myreq.dark_true = dark_real
myreq.masterdarkqc = mymasterdark.qc
myreq.make()
assert myreq.passed, "Requirement not met!"


print(dark_real, mymasterdark.qc.darkmed)

# ### Data Lineage
# The data lineage (or provenance) of the MasterDark can be explored.

mymasterdark.darks

mymasterdark.darks[0].raw.template.det__dit

[dark.raw.template.det__dit for dark in mymasterdark.darks]

# ### FITS keywords
# FITS keywords are generated automatically from the data lineage.

mydark.fits_keywords

mymasterdark.fits_keywords

# ScopeSIM input dictionaries can also be generated from the data lineage (for the Raw data). There is a rough one-to-one correspondence with the FITS keywords.

mydark.scopesim_dictionaries

# <a id = "Data items specification" DataItemsSpecification></a>
# ## Data items specification
# The full target diagram can be generated from the data lineage.

print(ditaa_from_wise(MasterDark))


