"""The Micado base class for Tasks (or recipes)"""

# import logging

# from common.database.support import oracleoidtype, OIDNONE
from common.database.DBMain import DBObject
from common.util.OptionsParser import OptionsParser
from common.util.Pipeline import Task as Task_common, TaskError


def serialize_object(obj):
    """A fixed dependency can be specified as object or object_id (hex and string)"""
    if isinstance(obj, DBObject):
        return obj.__class__, obj.object_id.binary()
    elif obj is None:
        return obj
    raise ValueError("Can not serialize; object %s" % obj)


def deserialize_object(key):
    """Deserialize (instantiate) the object"""
    if key:
        return key[0](object_id=key[1])
    return key


class DummyTarget:

    """A dummy target, this target will be used in Tasks or Recipes
    which do not define a target. A DummyTarget only defines a default
    processing time."""

    PROCESS_TIME = 60  # [seconds]


class Task(Task_common):

    target = DummyTarget

    # uncomment to disable cleanup of job working dir by dpu
    # dpu_debug_nocleanup = True

    # hook to communicate with the dpu server, will be set by the dpu
    dpu_conn = None

    # The options parser, derived classes must override this with actual options
    baseparser = OptionsParser(__name__)

    def __init__(self, *args, **kwargs):
        """initialize"""
        # parse the options
        super().__init__(*args, **kwargs)

        # if the user gave persistent objets, serialize
        to_del = []
        for key, val in self.options.items():
            if isinstance(val, DBObject):
                setattr(self, key, serialize_object(val))
                to_del.append(key)
        for key in to_del:
            del self.kwargs[key]
            del self.options[key]

        # will be set to the date execution started
        self.started = None

        # will contain the resulting products and errors
        self.process_errors = []

        # If there are persistent objects in self.args, self.kwargs or self.options
        # then these must be handled before sending them to the dpu
        # pickling persistent objects could take a long time, or even fail

    def get_process_time(self):
        """return the default processing time"""
        return 1799  # [seconds]

    def get_process_result(self):
        """return the processing result"""
        return self.process_result

    @classmethod
    def get_process_params(cls):
        """return the process parameters"""
        # TODO are the process parameters contained in the attribute called recipe ?
        if not hasattr(cls, "target"):
            raise TaskError(
                "Error; could not determine process parameters, Task (%s) has no target attribute"
                % cls.__name__
            )
        if not hasattr(cls.target, "process_params"):
            raise TaskError(
                "Error; could not determine process parameters, target (%s) has no process_params attribute"
                % cls.target.__name__
            )
        return cls.target().process_params

    def get_task_info(self):
        """return the task as info string, used for DPU_INFO"""
        # TODO; add info from the options
        return self.__class__.__name__
